# RandomLine.pl
#
# A Movable Type plugin tag for extracting and displaying a random line from
# an MT Module, or from a file.  Note that this randomness operates when 
# you rebuild a template with this tag in it--not when a user refreshes the 
# output page
#
# Author: Martin Sutherland (martin@sunpig.com)
#
# See http://www.sunpig.com/martin/projects/mt/randomline/ for documentation,
# installation instructions, and version history
#
# This is version 1.0 (28 Oct 2002)
#
# Notes:  This is the "getting it to work" version.  It's been a while
# since I've done any Perl, and it shows.  I know there are more efficient
# to write this code, but it's going to take me some time to figure them 
# out.  Future releases will be more optimised.
#
# Copyright (c) Sun Pig Enterprises 2002, but use it as you see fit.
#

use strict;
use MT::Template::Context;
MT::Template::Context->add_tag(RandomLine => sub { &randomLine; });


sub randomLine {

	# Get the context object, and the arguments for this tag
	my ($ctx, $args) = @_;

	# Check whether a file or a module has been specified.
	# If neither has been specified, return a blank.  If both have been
	# specified, use the module instead of the file.
	
	my @lines;
	# Check for module
	if ($args->{'module'}) {
		my $strModule = $args->{'module'};

		# Use the MT::Template class
		require MT::Template;

		# Load up the template module
		my $tmpl = MT::Template->load({ name => $strModule,
                                        blog_id => $ctx->stash('blog_id') })
            		or return $ctx->error("Unable to load the module '$strModule' for <MTRandomLine>");

		# Turn the text in the template into an array of all the lines therein
		my $text = $tmpl->text;

		# Turn the text into an array
		@lines = split /\n/, $text;
	}

	# Check for file
	if ($args->{'file'}) {
		my $strFile = $args->{'file'};

		# Try to open the file
		my $fh;
		open($fh, $strFile) or return $ctx->error("Unable to open the file '$strFile' for <MTRandomLine>");
		
		# Grab each line from the file, and add it to an array.
		my $line;
		while ($line = <$fh>) {
			push @lines, $line;
		}
	}

	
	# By default, the plugin will ignore blank lines in the file from which
	# the random line is to be extracted.  Under some circumstances, though,
	# a blank result may be an acceptable value.  If this isn't the case, 
	# we should remove blank lines from the array.
	
	# While we're at it, we should also strip newline characters from the end of each line.
	my ($line, @stripped);
	my $allowBlanks = $args->{'allowblanks'};

	while (@lines) {
		$line = shift @lines;

		# strip any newline or carriage return characters
		$line =~ s/(\n|\r)//g;

		unless ( ($line eq "") && !($allowBlanks) ) {
			push @stripped, $line;
		}
	}

	# Count the number of lines in the final array
	my $countLines = @stripped;

	# Get a random line from the array
	my $strLine = $stripped[int(rand $countLines)];

	# Return this value
	return ($strLine);
}